import re
import sys
import logging
import os.path as path


class Slide:
    """
    The Slide class defines a Slide used in a Presentation.  
    A slide is defined by its id, filename and title, and optionally a part number and a version number.
    The filenames points on a disk location where the slide content can be retrieved in a markdown format.
    The other arguments are:
    - title: arbitrary title
    - id: a unique integer. 2 slides can't have the same id, except if it is split.
    - part: [optional] float number. A Slide may be split in multiple parts. In this case, they have the same id but a different part number. If not set, 0.0 is the default value.
    - version: [optional] float number. A slide may have different versions, then a history may be managed (version 0 is older than version 1). If not set, 0.0 is the default value
    """

    def __init__(self, id: int, title: str, part:float=0.0, version:float=0.0, showLinks=True, isImage=False, display_title=False):
        """builds the object"""
        self.filename = None
        self.content = None
        self.isImage = isImage
        self.id = int(id)
        self.title = title
        self.part = float(part)
        self.version = float(version)
        self.showLinks = showLinks
        self.display_title = display_title

    def associateFile(self, filename: str):
        """
        associates a file to this slide to retrieve the content.
        A header may be defined in the markdown file. See The Slides object for more information.
        """
        self.filename = filename

    def setContent(self, text: str):
        """
        sets the content of the slide
        ---
        Parameters:
        - text: a markdown formated text
        """
        self.content = text

    def getContent(self):
        """ get slide content"""
        content = self.content
        if self.filename:
            if self.isImage:
                content = self._getImageContent()
            else:
                content = self._getFileContent()

        if self.display_title or not content:
            content = self._addTitle(content)

        return content

    def _getFileContent(self):
        """
        get markdown file content. 
        """
        with open(self.filename) as file:
            data = file.read()

        groups = re.split('---+', data)

        if len(groups) < 2:
            return data

        content = ''
        for group in groups[2:]:
            content += group
        return content

    def _getImageContent(self):
        # def createImageSlideContent(self, slides, outputFolder, image):
        imageLink = r'images/'+path.basename(self.filename)
        content = '![{0}]({1})'.format(
            self.title, re.sub(r'\s+', '%20', imageLink))
        return content

    def _addTitle(self, content):
        if not content:
            content = ''
        return '## '+self.title+'\n'+content



from pathlib import Path
from typing import List
import logging
import sys
import bisect as bs
from pyRevealjs.slide import Slide
from pyRevealjs.slide_generator import SlideGenerator


class SlideCatalog:
    """
    The SlideCatalog class manages a catalog of Slide that may be used by the Presentation object.

    The slides may be retrieved from a disk location if the markdown files defining the slide content respect one of the following rules:
    - the markdown file owns a header section composed of a "---" sequence defining the start and end of the header. It should be at the very beginning of the file.
    In this header at least 2 keys must be defined :
        - title: arbitrary title
        - id: a unique integer. 2 slides can't have the same id, except if it is split.
        - part: [optional] float number. A Slide may be split in multiple parts. In this case, they have the same id but a different part number. If not set, 0.0 is the default value.
        - version: [optional] float number. A slide may have different versions, then a history may be managed (version 0 is older than version 1). If not set, 0.0 is the default value
    - the markdown file name is built as follow: id_title[_part][_version]. The fields are the same than in the header definition
    The slides can also be added by passing Slide objects.
    The slides may be built thanks to images. Image names should comply with the same rule given for markdown file names.
    """

    def __init__(self):
        """builds the object"""
        self.slides = dict()
        """self.slides is a dictionary with key = slide id and value is the version dictionary.
        The version dictionary is defined by key = slide version and value is the part dictionary
        The part dictionary is defined by key = slide part number and value = Slide object"""
        self.versions = []
        """if True, the slide title is displayedin slides"""
        self.imageFolders = []

    def catalog(self, folder: str, images=False):
        """
        references slides by the files contained in the given folder if they comply with the rules defined in the class definition
        ---
        Parameters:
        - folder: folder where files to produce slides may be found
        - images: indicates if files are image files if True
        """
        logging.info('search for files to create slides...')

        if images:
            self.declareResources(folder)

        path = Path(folder).rglob('*.*')
        files = [x for x in path if x.is_file()]
        counter = 0
        for file in files:
            slide = None
            if images:
                slide = SlideGenerator().fromImage(file)
            else:
                slide = SlideGenerator().fromHeader(file)
                if not slide:
                    slide = SlideGenerator().fromFilename(file)
            if not slide:
                logging.warning(
                    'can\'t retrieve useful information from file {}, slide is not created.'.format(file))
                continue
            self.addSlide(slide)
            counter += 1
        if counter > 0:
            logging.info('{} slides created'.format(counter))
        else:
            logging.warning(
                'no file found to define slides in {}'.format(folder))

    def declareResources(self, imageFolder):
        self.imageFolders.append(imageFolder)

    def addSlide(self, slide: Slide):
        """add a predefined Slide object"""
        version = slide.version
        if slide.id not in self.slides:
            self.slides[slide.id] = dict()
        if version not in self.slides[slide.id]:
            bs.insort(self.versions, version)
            self.slides[slide.id][version] = dict()
        self.slides[slide.id][version][slide.part] = slide

    def createMissingSlides(self, slide_ids):
        """create missing slides according to the given slide ids. SlideCatalog are created with slide title="slide id" where id is the slide id.
        The slide is saved in the slideFolder with the markdown rules details in the class definition"""

        for slide_id in slide_ids:
            if self.getslide_parts(slide_id, self.getHighestVersion()):
                continue
            title = 'slide {}'.format(slide_id)
            slide = Slide(slide_id, title)
            slide.setContent('# '+title)
            self.addSlide(slide)
            logging.info('slide {} {} created'.format(slide_id, title))

    def getHighestVersion(self, slide_id=None):
        """returns the highest version value fourn for at least 1 slide in the collection"""
        if not self.versions:
            return None
        if not slide_id:
            return self.versions[-1]
        if slide_id not in self.slides:
            logging.error('no slide for id %s', slide_id)
            return None

        available_versions = sorted(list(self.slides[slide_id].keys()))

        return float(available_versions[-1])

    def showLinks(self, slide_id, version):
        slide_parts = self.getslide_parts(slide_id, version)
        show = True
        for part in slide_parts:
            show = show and slide_parts[part].showLinks
        return show

    def getslide_parts(self, slide_id, version=None):
        """returns a dictionary with key = part number and value = Slide object corresponding to the slide id and its version. 
        If no version is provided, latest version is used."""

        if not self.slides:
            logging.error("No slide in catalog")
            return None

        if slide_id not in self.slides:
            logging.error('slide {} not found'.format(slide_id))
            return None

        version = self._getSatisfyingVersion(slide_id, version)
        slide_id = int(slide_id)

        return self.slides[slide_id][version]

    def getslide_part(self, slide_id, part=None, version=None) -> Slide:
        """returns a slide part corresponding to the slide id, version and part. 
        If no part is provided, first part is used
        If no version is provided, lattest version is used."""

        slide_parts = self.getslide_parts(slide_id, version)
        if not slide_parts:
            return None

        part = float(part) if part else next(iter(slide_parts))

        return slide_parts[float(part)]

    def getSlideTitle(self, slide_id: int, version: float = None):
        """return the slide title by getting the slide title of its first part"""
        slide = self.getslide_part(slide_id, version=version)
        if not Slide:
            logging.error(
                'no slide for this id: %s / version: %s', slide_id, version)
        return slide.title

    def getSlideContents(self, slide_id: int, version: float = None)->List[str]:
        """returns slide contents, ie a list of markdown contents. Each item of the sorted list correspond to a slide part.If no version is provided, version 0.0 is returned."""
        contents = []
        slide_parts = self.getslide_parts(slide_id, version)
        if not slide_parts:
            contents.append('slide {} is missing.'.format(slide_id))
            return contents

        parts = sorted(list(slide_parts.keys()))

        for part in parts:
            slide_part:Slide = slide_parts[part]
            if slide_part is None:
                contents.append(
                    'slide {} part {} is missing.'.format(slide_id, part))
            contents.append(slide_part.getContent())

        return contents

    def getSlideImages(self, slide_ids: list = None, version: float = None) -> List[Slide]:
        """
        returns a list of filename of slides defined by an image
        Parameters:
        -----------
        slide_ids: [optional] ids of slide for which images should be retrieved, all will be retrieved if none is supplied
        version:  [optional] version of slides for which images should be retrieved. If None, the most recent version will be used.
        """
        if not slide_ids:
            slide_ids = self.getAllIds()

        slides = []
        for slide_id in slide_ids:

            version = self._getSatisfyingVersion(slide_id, version)

            for part in self.slides[slide_id][version]:
                slide = self.slides[slide_id][version][part]
                if slide.isImage:
                    slides.append(slide)

        return slides

    def getAllIds(self):
        return list(self.slides.keys())

    def _getSatisfyingVersion(self, slide_id: int, version:float)->float:

        if version is None:
            version = self.getHighestVersion(slide_id)
            index = self.versions.index(version)
            return version, index

        available_versions = sorted(list(self.slides[slide_id].keys()))
        satisfying_versions = list(
            filter(lambda i: i <= version, available_versions))

        if not satisfying_versions:
            satisfying_versions = [available_versions[0]]
            # TODO si aucune version plus petite ou égale à celle demandée, on prend la plus petite de la liste : ne devrait-on pas supprimer le slide ?
            # si on le supprime, alors la version slide et la version présentation sont cohérentes mais comment le gérer si le workflow et les liens y font appel ?

        version = float(satisfying_versions[-1])

        return version



class PresentationSettings():

    def __init__(self, settings:dict = {}):
        self.title = settings['title'] if 'title' in settings else 'Presentation'
        self.theme = settings['theme'] if 'theme' in settings else 'black'



from pyRevealjs import Slide, SlideCatalog, SlideGenerator, Presentation, PresentationSettings
import unittest
import tempfile
import logging
import sys


class testPresentation(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.temp_dir = tempfile.TemporaryDirectory()
        print('temp dir: ', cls.temp_dir.name)
        root = logging.getLogger()
        formatter = logging.Formatter(
            '%(asctime)s - %(levelname)s - %(filename)s - %(lineno)d - %(message)s')
        root.setLevel(logging.DEBUG)

        # console
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setLevel(logging.DEBUG)
        console_handler.setFormatter(formatter)
        root.addHandler(console_handler)

    @classmethod
    def tearDownClass(cls):
        cls.temp_dir.cleanup()

    def test_simple(self):
        # get slides from images
        catalog = SlideCatalog()
        catalog.catalog('tests/resources/onlyimages', images=True)
        # Create Presentation based on Slides in the current working directory
        settings = {'title': 'simple presentation', 'theme': 'white'}
        pres = Presentation('simple presentation', catalog, PresentationSettings(settings))
        pres.addSlideByIds(catalog.getAllIds())
        pres.save('temp')

    def test_full(self):
        # Create a slide with id=1. Part and version numbers are let to defaut values (0.0)
        # Content is given as a string
        slide1 = Slide(1, "slide 1")
        slide1.setContent(
            '# Slide1  \nversion {} - part {}'.format(slide1.version, slide1.part))

        # Create slide 2
        # Content is given as an external markdown file with header where id=2. Part and version numbers are let to defaut values (0.0)
        slide2part0 = SlideGenerator().fromHeader('tests/resources/full/slide2.md')

        # Create a slide with the same id than slide 2 because it is a second part (part number 1.1) of the same slide
        # Parts of a same slide will be displayed vertically in the presentation while different ids are displayed horizontally
        slide2part1 = Slide(2, 'slide2-1', part=1.1)
        slide2part1.setContent(
            '# Slide2  \nversion {} - part {}'.format(slide2part1.version, slide2part1.part))

        # The content is not defined, it will be automatically generated using the title of the slide
        # content is not defined for this slide, it will automatically generated based on its title
        slide3version0_1 = Slide(3, 'slide3', version=0.1)
        slide3version0_1.setContent(
            '# Slide3  \nversion {} - part {}'.format(slide3version0_1.version, slide3version0_1.part))

        slide3version1 = Slide(3, 'slide3', version=1)
        slide3version1.setContent(
            '# Slide3  \nversion {} - part {}'.format(slide3version1.version, slide3version1.part))

        # Add slides to Slides
        catalog = SlideCatalog()
        catalog.addSlide(slide1)
        catalog.addSlide(slide2part0)
        catalog.addSlide(slide2part1)
        catalog.addSlide(slide3version0_1)
        catalog.addSlide(slide3version1)

        # slide2.md asks to not display links
        links = {1: [2, 3], 2: [3]}

        # if slides embed images from imageFolder in their markdown content, the following line is required:
        # slides.declareResources(imageFolder):

        # Define versions of presentation to create
        # slides does not need to all have the same version. The presentation will find the closest version of slide less than the requested version if not found
        versions = [0.0, 0.1, 1.0]

        # Create Presentation based on Slides in the current working directory
        pres = Presentation('presentation', catalog)
        pres.addSlideByIds(catalog.getAllIds())
        pres.addLinks(links)
        temp = self.temp_dir.name
        for version in versions:
            pres.save('temp', version)



import os
import re
import sys
import logging
import pathlib
import tempfile
from bs4 import BeautifulSoup
from pyRevealjs.slide_catalog import SlideCatalog
from pyRevealjs.slide import Slide
from typing import List, Dict
import distutils.dir_util as dirutil
import distutils.file_util as fileutil
from pyRevealjs.presentation_settings import PresentationSettings


class Presentation:
    """
    The Presentation class writes on disk a presentation based on revealjs.
    It uses SlideCatalog class to retrieve the content to display in the presentation, 
    an ordered list of slide ids to create content in this given order. Optionaly it may use links to create 
    more complex presentatrion structure whe a slide ma points on multiple other slides via these links.
    Finally this presentation is versioned, it relies on the different slide versions.
    """

    def __init__(self, presName, catalog: SlideCatalog, settings: PresentationSettings = None):
        """
        prepare the presentation object with a Name and a path to write presentation on disk.
        ---
        Parameters:
        - presName: the filename of the presentation.
        - catalog: catalog of slide
        - settings: dictionary of general settings. If None, default are applied
        """
        self.presName = presName
        self.catalog = catalog
        self.settings = settings if settings else PresentationSettings()
        self.slide_ids = []
        self.links = None
        self.slide_order = None
        self.resource_folder = pathlib.Path(__file__).parent / 'assets'
        self.template =  self.resource_folder.joinpath('template.html')
        self.html: Dict[BeautifulSoup] = {}
        self.temp_dir = None
        self.files = {}

    def addLinks(self, links):
        """
        add links to certain slides.
        ---
        Parameters:
        - links: dictionary with key = current slide_id and values = next self.slide_order to be linked to current slide_id
        """
        self.links = links

    def addSlideById(self, id: int):
        self.slide_ids.append(id)

    def addSlideByIds(self, ids: List[int]):
        self.slide_ids.extend(ids)

    def defineSlideOrder(self, slide_order):
        """
        define in which order slides will be displayed
        ---
        Parameters:
        - slide_order: ordered list of Slide id. 
        the slide ids contained in this list must be consistent with the provided slide ids supplied with addSlide(s)
        slide_order can shorten the displayed slides but obviously can't point on non-referenced slides
        """
        self.slide_order = slide_order

    def generate(self, version: float = 0.0):
        """
        generate the html presentation.
        ---
        Parameters:
        - version: version of the presentation (float) = version of the slides if available or previous version if not. Default value is 0.0
        """
        logging.info('generate presentation {}, version {} '.format(
            self.presName, version))

        with open(self.template) as template:
            self.html[version] = BeautifulSoup(template, 'html.parser')

        self._setGeneralSettings(self.html[version])

        # if defineSlideOrder has not been called, all slides supplied with addSlide(s) are sorted increasingly and all are used
        if not self.slide_order:
            self.slide_order = sorted(self.slide_ids)

        for slide_id in self.slide_order:
            self._writeMarkdownSection(slide_id, version)

        self.temp_dir = tempfile.TemporaryDirectory().name
        output = os.path.join(self.temp_dir, str(version))
        file = self._prepareOutput(output, version)
        self.files[version] = file

        with open(file, "w") as output:
            output.write(self.html[version].prettify())

    def save(self, output_folder=None, version: float = 0.0):
        """
        save presentation in outpurFolder
        ---
        Parameters:
        - output_folder: folder where the presentation will be saved. If None, the current working directory (folder from where the app is launched) is used
        """
        if version not in self.html:
            self.generate(version)

        output_folder = output_folder if output_folder else os.getcwd()
        source_folder = os.path.join(self.temp_dir, str(version))
        # TODO best if we don't recopy existing files...
        dirutil.copy_tree(source_folder, output_folder)
        return self.files[version]

    def _prepareOutput(self, output_folder, version):
        pathlib.Path(output_folder).mkdir(parents=True, exist_ok=True)
        self._copyLibs(output_folder)
        self._copyImages(output_folder, version)
        return os.path.join(output_folder, "{}_v{}.html".format(self.presName, version))

    def _copyLibs(self, output_folder):
        logging.info('copying libs...')
        libs = self.resource_folder.joinpath('revealjsdist4.1.0')
        dirutil.copy_tree(libs, os.path.join(output_folder, 'libs'))

    def _copyImages(self, output_folder: str, version):
        logging.info('copying images...')
        output = os.path.join(output_folder, 'images')
        imageFolders = self.catalog.imageFolders
        self.catalog.getSlideImages(self.slide_ids, version)

        slide_images = self.catalog.getSlideImages(self.slide_ids, version)
        for slide in slide_images:
            filename = slide.filename
            new_file = self._copyImage(filename, output)
            slide.filename = new_file
        for folder in imageFolders:
            dirutil.copy_tree(folder, output, update=True)

    def _copyImage(self, filename, output):

        if output == os.path.dirname(filename):
            return filename

        if not os.path.exists(output):
            os.makedirs(output)

        newFile = os.path.join(output, os.path.basename(filename))
        fileutil.copy_file(filename, newFile, update=True)
        return newFile

    def _setGeneralSettings(self, html: BeautifulSoup):
        html.title.string = self.settings.title
        self._setTheme(html)

    def _setTheme(self, html: BeautifulSoup):
        theme_tag = html.find(id='theme')
        link = theme_tag['href']
        sep = r'/'
        split = link.rsplit(sep, 1)
        split[1] = self.settings.theme+'.css'
        link = sep.join(split)
        theme_tag['href'] = link

    def _writeMarkdownSection(self, slide_id, version):

        links = self._getMarkdownLinks(slide_id, version)
        slide_contents = self.catalog.getSlideContents(slide_id, version)
        showLinks = self.catalog.showLinks(slide_id, version)

        html: BeautifulSoup = self.html[version]
        slides = html.find(class_='slides')

        nb_of_part = len(slide_contents)
        if nb_of_part > 1:
            parent = html.new_tag("section")
            slides.append(parent)
        else:
            parent = slides

        for part, slide_content in enumerate(slide_contents):
            section = html.new_tag("section", id='id'+str(slide_id))
            section['data-markdown'] = ''
            text_area = html.new_tag("textarea")
            text_area['data-template'] = ''
            section.append(text_area)
            parent.append(section)

            if showLinks:
                line = self._getLinkLine(links, part, slide_contents)
                slide_content += line
            text_area.string = slide_content

    def _getLinkLine(self, links, part, slide_contents):

        if not links:
            return ''

        # add links only in the last part of the slide
        if part != len(slide_contents) - 1:
            return ''

        sep = '-'
        return "  \n"+sep.join(links)

    def _getMarkdownLinks(self, slide_id: int, version:float=None):
        """returns markdown links correponding to the given link ids. It transforms theses ids as html links in a Presentation."""
        if self.links is None:
            return None

        if slide_id not in self.links:
            return None

        mdLinks = []
        for id in self.links[slide_id]:
            href = '#/id'+str(id)
            text = self.catalog.getSlideTitle(id, version)
            mdLinks.append('['+text+']('+href+')')
        return mdLinks



from pyRevealjs.presentation_settings import PresentationSettings
from pyRevealjs.presentation import Presentation
from pyRevealjs.slide import Slide
from pyRevealjs.slide_generator import SlideGenerator
from pyRevealjs.slide_catalog import SlideCatalog



import logging
import os.path as path
import re
from pyRevealjs.slide import Slide


class SlideGenerator:
    """
    The SlideGenerator class tries to extract and returns needed information from files (markdown or images) to build a Slide object.
    It also checks that all necessary pieces of information are available to build this object.
    """

    def fromHeader(self, file):
        """
        extract information from header section in a file and returns a dictionary with the following key/value pairs:
        - title: arbitrary title
        - id: a unique integer. 2 slides can't have the same id, except if it is split.
        - part: [optional] float number. A Slide may be split in multiple parts. In this case, they have the same id but a different part number. If not set, 0.0 is the default value.
        - version: [optional] float number. A slide may have different versions, then a history may be managed (version 0 is older than version 1). If not set, 0.0 is the default value
        - displayLinks: [optional] indicate if links should be displayed for this slide  
        """
        with open(file) as f:
            data = f.read()

        contents = re.split('---+', data)

        if len(contents) < 2:
            return

        details = dict()
        header = contents[1]
        headerLines = header.splitlines()
        for line in headerLines:
            splitLine = line.split(':', 1)
            if len(splitLine) < 2:
                continue
            headerTag = splitLine[0].strip()
            details[headerTag] = splitLine[1].strip()

        return self._getSlide(details, file)

    def fromImage(self, file):
        """
        extract information from an image file name and returns a dictionary with the following key/value pairs:
        - title: arbitrary title
        - id: a unique integer. 2 slides can't have the same id, except if it is split.
        - part: [optional] float number. A Slide may be split in multiple parts. In this case, they have the same id but a different part number. If not set, 0.0 is the default value.
        - version: [optional] float number. A slide may have different versions, then a history may be managed (version 0 is older than version 1). If not set, 0.0 is the default value
        """
        details = self._fromFilename(file)
        return self._getSlide(details, file, isImage=True)

    def fromFilename(self, file):
        """
        extract information from the file name and returns a dictionary with the following key/value pairs:
        - title: arbitrary title
        - id: a unique integer. 2 slides can't have the same id, except if it is split.
        - part: [optional] float number. A Slide may be split in multiple parts. In this case, they have the same id but a different part number. If not set, 0.0 is the default value.
        - version: [optional] float number. A slide may have different versions, then a history may be managed (version 0 is older than version 1). If not set, 0.0 is the default value
        """
        details = self._fromFilename(file)
        return self._getSlide(details, file)

    def _fromFilename(self, file):
        filename = path.basename(file)
        patterns = re.search(r'([0-9]+)_([^_]+)_?([^_]*)_?(.*)\.',
                             filename, re.IGNORECASE)
        if not patterns or len(patterns.groups()) < 2:
            logging.warning(
                'filename {} does not comply with expected pattern: id_title[_part][_version]'.format(filename))
            return None

        details = dict()
        details['id'] = patterns.group(1)

        details['title'] = patterns.group(2)

        details['part'] = patterns.group(3) if len(
            patterns.group(3)) > 0 and patterns.group(3) else 0.0

        details['version'] = float(patterns.group(4)) if len(
            patterns.group(4)) > 0 and patterns.group(4) else 0.0

        return details

    def _getSlide(self, details, file, isImage=False):
        if not self._formatAndCheck(details):
            return None

        slide = Slide(details['id'], details['title'],
                      details['part'], details['version'], details['showlinks'], isImage=isImage, display_title=details['displaytitle'])
        slide.associateFile(file)
        return slide

    def _formatAndCheck(self, details: dict):
        """
        checks is the given details are sufficient to define a slide.
        It also defines default values for part and version if not present in details
        ---
        Parameters:
        - details: a dictionary containing information on slides.
        The check is ok if this dictionary contains at least the keys 'title' and 'id'
        and if the value associated with the id key is an integer value
        """

        if not details:
            return False

        keys = ['title', 'id']
        for key in keys:
            if key not in details:
                return False

            details['id'] = int(details['id'])
            details['part'] = float(
                details['part']) if 'part' in details else 0.0
            details['version'] = float(
                details['version']) if 'version' in details else 0.0
            details['showlinks'] = str(details['showlinks']).lower() in [
                'true', '1', 'y', 'yes', 'ok'] if 'showlinks' in details else False
            details['displaytitle'] = str(details['showlinks']).lower() in [
                'true', '1', 'y', 'yes', 'ok'] if 'showlinks' in details else False

        return True



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyRevealjs", 
    version="2.5",
    author="20centCroak",
    author_email="",
    description="revealjs based presentation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/20centcroak/pyRevealjs",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    include_package_data = True,
    install_requires=[
        'BeautifulSoup4'
    ]
)